﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Kresleni
{
    public partial class Area : UserControl
    {
        public Area()
        {
            InitializeComponent();
        }

        private int X0, Y0;
        private bool press = false;

        private Port[] ports = null;

        private void createPorts()
        {
            if (ports == null)
            {
                ports = new Port[4];
                for (int i = 0; i <= 3; i++)
                {
                    Port p = new Port();
                    p.Parent = this.Parent;
                    p.Parent.Controls.SetChildIndex(p, 0); // put above
                    p.PortMove += portMoved;
                    ports[i] = p;
                }
                movePorts();
            }
        }

        private void movePorts()
        {
            if (ports != null)
            {
                int w0 = ports[0].Width / 2;
                int h0 = ports[0].Height / 2; ;
                int x = this.Left;
                int y = this.Top;
                int h = this.Height;
                int w = this.Width;
                for (int i = 0; i <= 3; i++)
                {
                    Port p = ports[i];

                    if (i == 1 || i == 2)
                        p.Left = x + w - w0;
                    else
                        p.Left = x - w0;

                    if (i == 2 || i == 3)
                        p.Top = y + h - h0;
                    else
                        p.Top = y - h0;
                }
            }
        }

        private void portMoved (Port p, int deltax, int deltay)
        {
            int i = 0;
            for (int k = 0; k < 4; k++)
                if (p == ports[k])
                    i = k;

            if (i == 0)
            {
                Left += deltax;
                Top += deltay;
                movePorts();
            }
            else if (i == 1)
            {
                Width += deltax;
                Top += deltay;
                movePorts();
            }
            else if (i == 2)
            {
                Width += deltax;
                Height += deltay;
                movePorts();
            }
            else if (i == 3)
            {
                Left += deltax;
                Height += deltay;
                movePorts();
            }
        }

        private void Area_MouseDown(object sender, MouseEventArgs e)
        {
            X0 = e.X;
            Y0 = e.Y;
            press = true;

            createPorts();
        }

        private void Area_MouseMove(object sender, MouseEventArgs e)
        {
            if (press)
            {
                if (e.Button == MouseButtons.Left)
                {
                    Left += e.X - X0;
                    Top += e.Y - Y0;
                    movePorts();
                }
                else if (e.Button == MouseButtons.Right)
                {
                    Width += e.X - X0;
                    Height += e.Y - Y0;
                    X0 = e.X;
                    Y0 = e.Y;
                    movePorts();
                }
                else if (e.Button == MouseButtons.Middle)
                {
                    colorDialog1.Color = BackColor;
                    if (colorDialog1.ShowDialog() == DialogResult.OK)
                        BackColor = colorDialog1.Color;
                }
            }
        }

        private void Area_MouseUp(object sender, MouseEventArgs e)
        {
            press = false;
        }

        private void Load_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                BackgroundImage = new Bitmap(openFileDialog1.FileName);
            }
        }

        private void Stretch_Click(object sender, EventArgs e)
        {
            if (BackgroundImageLayout == ImageLayout.Stretch)
            {
                BackgroundImageLayout = ImageLayout.Tile;
                stretchMenuItem.Checked = false;
            }
            else
            {
                BackgroundImageLayout = ImageLayout.Stretch;
                stretchMenuItem.Checked = true;
            }
        }


    }
}
